@@ -21,11 +21,12 @@ from pandas import Series, DataFrame
 import pandas as pd
 import numpy as np
 
-from .read.dinogws import DinoGws
-from .plots.plotheads import PlotHeads
+##from .read import dinogws
+from .plots import plotheads as plotheadsmodule
 from .stats.gxg import GxgStats
 from .stats.gwtimestats import GwTimeStats
 
+
 class GwSeries:
     """ 
     Groundwater heads time series management
@@ -84,7 +85,7 @@ class GwSeries:
 
     Notes
     -----
-    Head measurements are stored in meters relatieve to welltopStores
+    Head measurements are stored in meters relative to welltop 
     and served in several units: mwelltop,mref,msurfacelevel.
 
     Valid row names for locprops and column names for tubeprops are
@@ -122,34 +123,6 @@ class GwSeries:
         'datum','surface','mp',
         ]
 
-    _mapping_dinoheadprops = OrderedDict([
-        ("headdatetime","peildatum"),("headmp","standcmmp"),
-        ("headnote","bijzonderheid"),("remarks","opmerking"),
-        ])
-
-    _mapping_dinolocprops = OrderedDict([
-        ('locname','nitgcode'),
-        ('filname','filter'),
-        ('alias','tnocode'),
-        ('xcr','xcoor'),
-        ('ycr','ycoor'),
-        ('height_datum','NAP'),
-        ('grid_reference','RD'),
-        ])
-
-    _mapping_dinotubeprops = OrderedDict([
-        ('startdate','startdatum'),
-        ('mplevel','mpcmnap'),
-        ('filtop','filtopcmnap'),
-        ('filbot','filbotcmnap'),
-        ('surfacedate','mvdatum'),
-        ('surfacelevel','mvcmnap'),
-        ])
-
-
-    def __repr__(self):
-        return (f'{self.name()} (n={len(self._heads)})')
-
 
     def __init__(self,heads=None,locprops=None,tubeprops=None):
         """
@@ -192,6 +165,11 @@ class GwSeries:
         else:
             raise TypeError(f'heads is not a pandas DataFrame but {type(heads)}')
 
+    def __repr__(self):
+        return (f'{self.name()} (n={len(self)})')
+
+    def __len__(self):
+        return len(self._heads)
 
     def _validate_reference(self,ref):
 
@@ -206,69 +184,6 @@ class GwSeries:
 
         return ref
 
-
-    @classmethod
-    def from_dinogws(cls,filepath):
-        """ 
-        Read tno dinoloket csvfile with groundwater measurements and return data as gwseries object
-
-        Parameters
-        ----------
-        filepath : str
-            path to dinocsv file with measured groundwater heads
-
-        Returns
-        -------
-        result : GwSeries object
-
-        Example
-        -------
-        gw = GwSeries.from_dinogws(<filepath>)
-        jsondict = gw.to_json(<filepath>)
-        gw.from_json(<filepath>)
-        
-        """
-
-        # read dinofile to DinoGws object
-        ##dn = GwSeries.read.dinogws.DinoGws(filepath=filepath)
-        dn = DinoGws(filepath=filepath)
-
-        dinoprops = list(dn.header().columns)
-
-        # get location metadata
-        locprops = Series(index=cls._locprops_names,dtype='object')
-
-        for propname in cls._locprops_names:
-            dinoprop = cls._mapping_dinolocprops[propname]
-            if dinoprop in dinoprops:
-                locprops[propname] = dn.header().at[0,dinoprop]
-
-        locprops['grid_reference'] = 'RD'
-        locprops['height_datum'] = 'mNAP'
-        locprops = Series(locprops)
-
-        # get piezometer metadata
-        tubeprops = DataFrame(columns=cls._tubeprops_names)
-        for prop in cls._tubeprops_names:
-            dinoprop = cls._mapping_dinotubeprops[prop]
-            if dinoprop in dinoprops:
-                tubeprops[prop] = dn.header()[dinoprop]
-
-        for col in cls._tubeprops_numcols:
-                tubeprops[col] = pd.to_numeric(tubeprops[col],
-                                 errors='coerce')/100.
-
-        # get head measurements
-        dinoprops = list(dn.headdata().columns)
-        heads = DataFrame(columns=cls._headprops_names)
-        for prop in cls._headprops_names:
-            dinoprop = cls._mapping_dinoheadprops[prop]
-            if dinoprop in dinoprops:
-                heads[prop] = dn.headdata()[dinoprop]
-        heads['headmp'] = heads['headmp']/100.
-
-        return cls(heads=heads,locprops=locprops,tubeprops=tubeprops)
-
     @classmethod
     def from_json(cls,filepath=None):
         """ Read gwseries object from json file """
@@ -291,17 +206,17 @@ class GwSeries:
         return cls(heads=heads,locprops=locprops,tubeprops=tubeprops)
 
 
-    def to_json(self,dirpath=None):
+    def to_json(self,filepath=None):
         """ 
         Create json string from GwWeries object and optionally write 
         to file.
 
         Parameters
         ----------
-        dirpath : str
-           directory json file will be written to
-           (if dirpath is not given no textfile will be written and 
-           only OrderedDict with valid JSON wil be retruned)
+        path : str
+           Filepath or directory the json file will be written to.
+           If filepath is not given no textfile will be written and 
+           only OrderedDict with valid JSON wil be retruned.
 
         Returns
         -------
@@ -331,16 +246,17 @@ class GwSeries:
         json_dict['heads'] = json_heads
         json_formatted_str = json.dumps(json_dict, indent=2)
 
-        if isinstance(dirpath,str):
-            try:
-                filepath = os.path.join(dirpath,self.name()+'.json')
-                with open(filepath,"w") as f:
-                    f.write(json_formatted_str)
-            except FileNotFoundError:
-                print("Filepath {} does not exist".format(filepath))
-                return None
-            #finally:
-            #    json_dict
+        if os.path.isdir(filepath):
+            filepath = os.path.join(filepath,self.name()+'.json')
+
+        try:            
+            with open(filepath,"w") as f:
+                f.write(json_formatted_str)
+        except FileNotFoundError:
+            print("Filepath {} does not exist".format(filepath))
+            return None
+        #finally:
+        #    json_dict
 
         return json_dict
 
@@ -401,30 +317,16 @@ class GwSeries:
 
     def locname(self):
         """Return series location name"""
-        srname = self.locprops().index[0]
-        locname = self.locprops().loc[srname,'locname']
-        return locname
+        ##srname = self.locprops().index[0]
+        ##locname = self.locprops().loc[srname,'locname']
+        return self._locprops['locname']
 
 
-    def locprops(self,minimal=False):
-        """
-        Return location properties as pd.DataFrame.
-
-        Parameters
-        ---------=
-        minimal : bool, default=False
-            return only minimal selection of columns
-
-        Returns
-        -------
-        pd.DataFrame
-        """
-
-        sr = self._locprops
-        sr.name = self.name()
-        locprops = DataFrame(sr).T
-        if minimal:
-            locprops = locprops[self._locprops_minimal]
+    def locprops(self):
+        """Return location properties as pd.DataFrame."""
+        locprops = self._locprops
+        locprops = locprops.drop('filname')
+        locprops = DataFrame(locprops).T.set_index('locname')
         return locprops
 
 
@@ -644,7 +546,7 @@ class GwSeries:
         return sr
 
 
-    def tubeprops_changes(self,proptype='mplevel'):
+    def tubeprops_changes(self,proptype='mplevel',relative=True):
         """
         Return timeseries with tubeprops changes.
 
@@ -652,6 +554,8 @@ class GwSeries:
         ----------
         proptype : ['mplevel','surfacelevel','filtop','filbot'
             Tubeproperty that is shown in reference cange graph.
+        relative : bool, default True
+            Changes relative to first value.
 
         Returns
         -------
@@ -672,7 +576,8 @@ class GwSeries:
         sr2 = Series(mps,index=idx)
 
         sr12 = pd.concat([sr1,sr2]).sort_index()
-        sr12 = sr12 - sr12[0]
+        if relative:
+            sr12 = sr12 - sr12[0]
 
         return sr12
 
@@ -690,14 +595,16 @@ class GwSeries:
         if proptype in ['mplevel','surfacelevel','filtop','filbot']:
             ##mps = self._tubeprops[proptype].values
             mps = self.tubeprops_changes(proptype=proptype)
-            self.headsplot = PlotHeads(ts=[self.heads()],mps=mps)
+            self.headsplot = plotheadsmodule.PlotHeads(ts=[self.heads()],mps=mps)
 
         if proptype is None:
-            self.headsplot = PlotHeads(ts=[self.heads()])
+            self.headsplot = plotheadsmodule.PlotHeads(ts=[self.heads()])
 
         if filename is not None:
             self.headsplot.save(filename)
 
+        return self.headsplot
+
 
     def gxg(self,ref='datum',minimal=True,name=True):
         """
@@ -717,6 +624,8 @@ class GwSeries:
         gxg : pd.Series
             gxg descriptive statistics
         """
+        # Calculating gxg can take considerable time. Therefore,
+        # results are stored.
         if not hasattr(self,'_gxg'):
             self._gxg = GxgStats(self)            
 
@@ -746,3 +655,65 @@ class GwSeries:
         xg = self._gxg.xg(reference=ref,name=name)
 
         return xg
+
+    def get_quantiles(self, ref='surface', unit='days', step=None):
+        """Calculate quantiles of measured heads.
+
+        Parameters
+        ----------
+        headsref : {'datum','surface'}, default 'surface'
+            Reference level for measurements.
+        unit : {'days','quantiles'}, default 'days'
+            Unit of quantile boundary classes.
+        step : float or int
+            Quantile class division steps. For unit days an integer 
+            between 0 and 366, for unit quantiles a fraction between 
+            0 and 1.
+
+        Returns
+        -------
+        pandas.DataFrame
+        """
+        # bypass circular import
+        from .stats.quantiles import Quantiles
+
+        qt = Quantiles(self.heads(ref=ref))
+        return qt.get_quantiles()
+
+    def get_ecostats(self, ref='surface', units='days', step=5):
+        """Return ecological most relevant statistics.
+
+        Parameters
+        ----------
+        ref : {'datum','surface'}, default 'surface'
+            Reference level for measurements.
+        units : {'days','quantiles'}, default 'days'
+            Unit of quantile boundary classes.
+        step : float or int, default 5
+            Quantile class division steps. For unit days an integer 
+            between 0 and 366, for unit quantiles a fraction between 
+            0 and 1.
+
+        Returns
+        -------
+        pandas.Series      
+        """
+
+        # bypass circular import
+        from .stats.quantiles import Quantiles
+        
+        qt = Quantiles(self.heads(ref=ref))
+        inundation = qt.get_inundation()
+        lowest = qt.get_lowest()
+
+        ecostats = self.gxg(ref='surface',minimal=True)
+       
+        ecostats['lowest_mean'] = lowest['mean']
+        ecostats['lowest_min'] = lowest['min']
+        ecostats['lowest_max'] = lowest['max']
+
+        ecostats['inundation_mean'] = inundation['mean']
+        ecostats['inundation_min'] = inundation['min']            
+        ecostats['inundation_max'] = inundation['max']
+
+        return ecostats
\ No newline at end of file
